@@ -8,7 +8,7 @@
 #' @param dropout_model The dropout model used to analyze the dropout data
 #'   which can be set to one of the following options:
 #'   "exponential", "Weibull", "log-logistic", "log-normal",
-#'   "piecewise exponential", "model averaging", or "spline".
+#'   "piecewise exponential", "model averaging", "spline", or "cox model".
 #'   The model averaging uses the \code{exp(-bic/2)} weighting and
 #'   combines Weibull and log-normal models. The spline model of
 #'   Royston and Parmar (2002) assumes that a transformation of
@@ -28,6 +28,8 @@
 #'   as a spline function. If "odds", the log cumulative odds is
 #'   modeled as a spline function. If "normal", -qnorm(S(t)) is
 #'   modeled as a spline function.
+#' @param m_dropout The number of dropout time intervals to extrapolate
+#'   the hazard function beyond the last observed dropout time.
 #' @param showplot A Boolean variable to control whether or not to
 #'   show the fitted time-to-dropout survival curve. By default, it is
 #'   set to \code{TRUE}.
@@ -54,6 +56,27 @@
 #' If the spline option is chosen, the \code{knots} and \code{scale}
 #' will be included in the list of results.
 #'
+#' If the cox model option is chosen, the list of results will include
+#' \code{model}, \code{theta}, \code{vtheta}, \code{aic}, \code{bic}, and
+#' \code{piecewiseDropoutTime}. Here
+#' \deqn{\theta = (\log(\lambda_1), \ldots, \log(\lambda_M), \beta^T)^T,}
+#' \eqn{M} denotes the number of distinct observed dropout times,
+#' \eqn{t_1 < \cdots < t_M},
+#' \eqn{\lambda_j} denotes the estimated hazard rate in the \eqn{j}th
+#' dropout time interval, \eqn{(t_{j-1}, t_j]}, and
+#' \eqn{\beta} represents the regression
+#' coefficients (log hazard ratios) from the Cox model.
+#' For a fair comparison, the estimation of baseline hazards is
+#' incorporated into the \code{aic} and \code{bic} values.
+#' In addition, \eqn{\mbox{piecewiseDropoutTime} = (0, t_1, \ldots, t_M)}.
+#' To extend the survival curve
+#' beyond the last observed dropout time, a weighted average of the hazard
+#' rates from the final \code{m_dropout} dropout time intervals is used.
+#' The weights are proportional to the lengths of those intervals, i.e.,
+#' \deqn{\lambda_{M+1} = \sum_{j=M-m_{\rm{dropout}}+1}^{M} w_j \lambda_j,}
+#' where \eqn{w_j = (t_j - t_{j-1})/(t_M - t_{M-m_{\rm{dropout}}})} for
+#' \eqn{j=M-m_{\rm{dropout}}+1,\ldots,M}.
+#'
 #' When fitting the dropout model by treatment, the outcome is presented
 #' as a list of lists, where each list element corresponds to a
 #' specific treatment group.
@@ -70,14 +93,16 @@
 #'
 #' @examples
 #'
-#' dropout_fit <- fitDropout(df = interimData2,
-#'                           dropout_model = "exponential")
+#' dropout_fit <- fitDropout(
+#'   df = interimData2,
+#'   dropout_model = "exponential")
 #'
 #' @export
 #'
 fitDropout <- function(df, dropout_model = "exponential",
                        piecewiseDropoutTime = 0,
                        k_dropout = 0, scale_dropout = "hazard",
+                       m_dropout = 5,
                        showplot = TRUE, by_treatment = FALSE,
                        covariates = NULL) {
 
@@ -86,7 +111,7 @@ fitDropout <- function(df, dropout_model = "exponential",
   erify::check_content(tolower(dropout_model),
                        c("exponential", "weibull", "log-logistic",
                          "log-normal", "piecewise exponential",
-                         "model averaging", "spline"))
+                         "model averaging", "spline", "cox model"))
 
   if (piecewiseDropoutTime[1] != 0) {
     stop("piecewiseDropoutTime must start with 0");
@@ -98,6 +123,7 @@ fitDropout <- function(df, dropout_model = "exponential",
 
   erify::check_n(k_dropout, zero = TRUE)
   erify::check_content(tolower(scale_dropout), c("hazard", "odds", "normal"))
+  erify::check_n(m_dropout)
 
   erify::check_bool(showplot)
   erify::check_bool(by_treatment)
@@ -115,20 +141,18 @@ fitDropout <- function(df, dropout_model = "exponential",
     formula = survival::Surv(time, dropout) ~ 1
   }
 
-
-  df <- dplyr::as_tibble(df)
-  names(df) <- tolower(names(df))
+  dt <- data.table::setDT(data.table::copy(df))
 
   if (by_treatment) {
-    ngroups = length(table(df$treatment))
+    ngroups = dt[, data.table::uniqueN(get("treatment"))]
 
-    if (!("treatment_description" %in% names(df))) {
-      df <- df %>% dplyr::mutate(
-        treatment_description = paste("Treatment", .data$treatment))
+    if (!("treatment_description" %in% names(dt))) {
+      dt[, `:=`(treatment_description =
+                  paste("Treatment", get("treatment")))]
     }
   } else {
     ngroups = 1
-    df <- df %>% dplyr::mutate(treatment = 1)
+    dt[, `:=`(treatment = 1)]
   }
 
   if (ngroups == 1) {
@@ -140,19 +164,19 @@ fitDropout <- function(df, dropout_model = "exponential",
   dropout_fit <- list()
 
   for (i in 1:ngroups) {
-    df1 <- df %>% dplyr::filter(.data$treatment == i)
+    df1 <- dt[get("treatment") == i]
 
     n0 = nrow(df1)
-    c0 = sum(df1$dropout)
-    ex0 = sum(df1$time)
+    c0 = df1[, sum(get("dropout"))]
+    ex0 = df1[, sum(get("time"))]
 
     x = model.matrix(formula, df1)
     q = ncol(x) - 1
 
     kmfit <- survival::survfit(survival::Surv(time, dropout) ~ 1, data = df1)
-    kmdf <- dplyr::tibble(time = kmfit$time, surv = kmfit$surv)
-    kmdf <- dplyr::tibble(time = 0, surv = 1) %>%
-      dplyr::bind_rows(kmdf)
+    kmdf <- data.table(time = kmfit$time, surv = kmfit$surv)
+    df0 <- data.table(time = 0, surv = 1)
+    kmdf <- data.table::rbindlist(list(df0, kmdf), use.names = TRUE)
 
     if (tolower(dropout_model) == "exponential") {
       erify::check_positive(c0 - q, supplement = paste(
@@ -174,11 +198,9 @@ fitDropout <- function(df, dropout_model = "exponential",
       # fitted survival curve
       rate = exp(as.numeric(x %*% fit3$theta))
 
-      dffit3 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(pexp(t, rate, lower.tail = FALSE))))
-
+      dffit3 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(pexp(t, rate, lower.tail = FALSE))))]
     } else if (tolower(dropout_model) == "weibull") {
       erify::check_positive(c0 - q - 1, supplement = paste(
         "The number of dropouts must be >=", q + 2,
@@ -202,11 +224,9 @@ fitDropout <- function(df, dropout_model = "exponential",
       shape = exp(-fit3$theta[q+2])
       scale = exp(as.numeric(x %*% fit3$theta[1:(q+1)]))
 
-      dffit3 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(pweibull(t, shape, scale, lower.tail = FALSE))))
-
+      dffit3 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(pweibull(t, shape, scale, lower.tail = FALSE))))]
     } else if (tolower(dropout_model) == "log-logistic") {
       erify::check_positive(c0 - q - 1, supplement = paste(
         "The number of dropouts must be >=", q + 2,
@@ -229,11 +249,9 @@ fitDropout <- function(df, dropout_model = "exponential",
       location = as.numeric(x %*% fit3$theta[1:(q+1)])
       scale = exp(fit3$theta[q+2])
 
-      dffit3 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(plogis(log(t), location, scale, lower.tail = FALSE))))
-
+      dffit3 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(plogis(log(t), location, scale, lower.tail = FALSE))))]
     } else if (tolower(dropout_model) == "log-normal") {
       erify::check_positive(c0 - q - 1, supplement = paste(
         "The number of dropouts must be >=", q + 2,
@@ -244,7 +262,7 @@ fitDropout <- function(df, dropout_model = "exponential",
       reg <- survival::survreg(formula, data = df1, dist = "lognormal")
 
       # we use parameterization theta = (meanlog, log(sdlog))
-      # reg$var is for c(reg$coefficients, log(reg$scale)) = theta
+      # reg$var is for theta = c(reg$coefficients, log(reg$scale))
       fit3 <- list(model = "Log-normal",
                    theta = c(as.numeric(reg$coefficients), log(reg$scale)),
                    vtheta = reg$var,
@@ -255,14 +273,13 @@ fitDropout <- function(df, dropout_model = "exponential",
       meanlog = as.numeric(x %*% fit3$theta[1:(q+1)])
       sdlog = exp(fit3$theta[q+2])
 
-      dffit3 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(plnorm(t, meanlog, sdlog, lower.tail = FALSE))))
-
+      dffit3 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(plnorm(t, meanlog, sdlog, lower.tail = FALSE))))]
     } else if (tolower(dropout_model) == "piecewise exponential") {
-      # lambda(t) = lambda[j] for ucut[j] < t <= ucut[j+1], j = 1,...,J
-      # where ucut[1]=0< ucut[2] < ... < ucut[J] < ucut[J+1]=Inf are the knots
+      # lambda_0(t) = lambda[j] for ucut[j] <= t < ucut[j+1], j = 1,...,J
+      # where ucut[1]=0< ucut[2] < ... < ucut[J] < ucut[J+1]=Inf are
+      # the knots
       J = length(piecewiseDropoutTime)
 
       erify::check_positive(c0 - J - q + 1, supplement = paste(
@@ -277,11 +294,10 @@ fitDropout <- function(df, dropout_model = "exponential",
 
       surv = purrr::map(1:n0, function(l)
         ppwexp(time, fit3$theta, J, fit3$piecewiseDropoutTime,
-               q, x[l,-1], lower.tail = FALSE))
+               q, x[l,], lower.tail = FALSE))
       surv = apply(matrix(purrr::list_c(surv), ncol = n0), 1, mean)
 
-      dffit3 <- dplyr::tibble(time, surv)
-
+      dffit3 <- data.table(time, surv)
     } else if (tolower(dropout_model) == "model averaging") {
       erify::check_positive(c0 - q - 1, supplement = paste(
         "The number of dropouts must be >=", q + 2,
@@ -317,11 +333,10 @@ fitDropout <- function(df, dropout_model = "exponential",
       time = seq(0, max(df1$time))
 
       surv = purrr::map(1:n0, function(l)
-        pmodavg(time, fit3$theta, w1, q, x[l,-1], lower.tail = FALSE))
+        pmodavg(time, fit3$theta, w1, q, x[l,], lower.tail = FALSE))
       surv = apply(matrix(purrr::list_c(surv), ncol = n0), 1, mean)
 
-      dffit3 <- dplyr::tibble(time, surv)
-
+      dffit3 <- data.table(time, surv)
     } else if (tolower(dropout_model) == "spline") {
       erify::check_positive(c0 - k_dropout - q - 1, supplement = paste(
         "The number of dropouts must be >=", k_dropout + q + 2,
@@ -334,10 +349,10 @@ fitDropout <- function(df, dropout_model = "exponential",
         method = "Nelder-Mead")
 
       fit3 <- list(model = "Spline",
-                   theta = spl$coefficients,
+                   theta = as.numeric(spl$coefficients),
                    vtheta = spl$cov,
-                   aic = -2*spl$loglik + 2*(k_dropout+2),
-                   bic = -2*spl$loglik + (k_dropout+2)*log(n0),
+                   aic = -2*spl$loglik + 2*(k_dropout+q+2),
+                   bic = -2*spl$loglik + (k_dropout+q+2)*log(n0),
                    knots = spl$knots,
                    scale = spl$scale)
 
@@ -359,7 +374,81 @@ fitDropout <- function(df, dropout_model = "exponential",
           lower.tail = FALSE)
       }
 
-      dffit3 <- dplyr::tibble(time, surv)
+      dffit3 <- data.table(time, surv)
+    } else if (tolower(dropout_model) == "cox model") {
+      erify::check_positive(c0 - q, supplement = paste(
+        "The number of dropouts must be >=", q + 1,
+        "to fit a Cox model."))
+
+      erify::check_positive(c0 - m_dropout + 1, supplement = paste(
+        "m_dropout must be <= the observed number of dropouts", c0))
+
+      reg <- phregr(df1, time = "time", event = "dropout",
+                    covariates = covariates)
+
+      bh <- data.table::setDT(reg$basehaz)[get("nevent") > 0]
+      d <- bh$nevent
+      llik <- reg$sumstat$loglik1 + sum(d*(log(d) - 1))
+
+      haz <- bh$haz
+      vhaz <- bh$varhaz
+
+      if (q > 0) {
+        if (q == 1) {
+          ghaz <- matrix(bh$gradhaz, ncol = 1)
+        } else {
+          ghaz <- do.call(cbind, lapply(1:q, function(i)
+            bh[[paste0("gradhaz.", i)]]))
+        }
+      }
+
+      M <- nrow(bh)
+      tcut <- c(0, bh$time)
+      lambda1 <- haz/diff(tcut)
+
+      if (q > 0) {
+        theta <- c(log(lambda1), as.numeric(reg$beta))
+        vbeta <- reg$vbeta
+        dimnames(vbeta) <- NULL
+        vtheta <- matrix(0, M+q, M+q)
+        vtheta[(M+1):(M+q),(M+1):(M+q)] <- vbeta
+        vtheta[1:M,1:M] <- diag(vhaz/(haz*haz)) + ghaz %*% vbeta %*% t(ghaz)
+        vtheta[1:M,(M+1):(M+q)] <- ghaz %*% vbeta
+        vtheta[(M+1):(M+q),1:M] <- t(vtheta[1:M,(M+1):(M+q)])
+      } else {
+        theta <- log(lambda1)
+        vtheta <- diag(vhaz/(haz*haz))
+      }
+
+      # account for the estimation of baseline hazard in AIC and BIC
+      fit3 <- list(model = "Cox model",
+                   theta = theta,
+                   vtheta = vtheta,
+                   aic = -2*llik + 2*(M+q),
+                   bic = -2*llik + (M+q)*log(n0),
+                   piecewiseDropoutTime = tcut)
+
+      # fitted survival curve
+      time = seq(0, max(df1$time))
+
+      # extrapolate beyond the last observed dropout time
+      lambda2 <- sum(bh$haz[(M-m_dropout+1):M])/
+        (bh$time[M] - bh$time[M-m_dropout])
+
+      lambda <- c(lambda1, lambda2)
+
+      # baseline survival
+      s1 <- sapply(time, function(t)
+        ppwexp(t, log(lambda), M+1, tcut, lower.tail = FALSE))
+
+      if (q > 0) {
+        xbeta <- as.numeric(as.matrix(x[,-1]) %*% reg$beta)
+        surv <- apply(outer(s1, exp(xbeta), `^`), 1, mean)
+      } else {
+        surv <- s1
+      }
+
+      dffit3 <- data.table(time, surv)
     }
 
 
@@ -370,6 +459,8 @@ fitDropout <- function(df, dropout_model = "exponential",
     } else if (tolower(fit3$model) == "spline") {
       modeltext = paste0(fit3$model, "(k = ", k_dropout, ", ", "scale = '",
                          scale_dropout, "')")
+    } else if (tolower(fit3$model) == "cox model") {
+      modeltext = paste0(fit3$model, "(m = ", m_dropout, ")")
     } else {
       modeltext = fit3$model
     }
@@ -402,16 +493,14 @@ fitDropout <- function(df, dropout_model = "exponential",
           showarrow = FALSE)) %>%
       plotly::hide_legend()
 
-    if (by_treatment && ngroups > 1) {
+    if (by_treatment) {
       fittedDropout <- fittedDropout %>%
         plotly::layout(annotations = list(
           x = 0.5, y = 1,
           text = paste0("<b>", df1$treatment_description[1], "</b>"),
           xanchor = "center", yanchor = "middle", showarrow = FALSE,
           xref = 'paper', yref = 'paper'))
-    }
 
-    if (by_treatment) {
       fit3$treatment = df1$treatment[1]
       fit3$treatment_description = df1$treatment_description[1]
     }
@@ -423,13 +512,15 @@ fitDropout <- function(df, dropout_model = "exponential",
 
   # ensure that the sub plots share the same x axis range
   if (by_treatment) {
-    x_range = range(df$time)
+    x_range = range(dt$time)
     for (i in 1:ngroups) {
       dropout_fit[[i]]$fit_plot <- dropout_fit[[i]]$fit_plot %>%
         plotly::layout(xaxis = list(range = x_range))
     }
   } else {
-    dropout_fit = list(fit = fit3, fit_plot = fittedDropout)
+    dropout_fit = list(fit = fit3, fit_plot = fittedDropout,
+                       kmdf = kmdf, dffit = dffit3,
+                       text = c(modeltext, aictext, bictext))
   }
 
   if (showplot) {
