@@ -8,7 +8,8 @@
 #' @param event_model The event model used to analyze the event data
 #'   which can be set to one of the following options:
 #'   "exponential", "Weibull", "log-logistic", "log-normal",
-#'   "piecewise exponential", "model averaging", or "spline".
+#'   "piecewise exponential", "model averaging", "spline", or
+#'   "cox model".
 #'   The model averaging uses the \code{exp(-bic/2)} weighting and
 #'   combines Weibull and log-normal models. The spline model of
 #'   Royston and Parmar (2002) assumes that a transformation of
@@ -28,6 +29,8 @@
 #'   as a spline function. If "odds", the log cumulative odds is
 #'   modeled as a spline function. If "normal", -qnorm(S(t)) is
 #'   modeled as a spline function.
+#' @param m The number of event time intervals to extrapolate the hazard
+#'   function beyond the last observed event time.
 #' @param showplot A Boolean variable to control whether or not to
 #'   show the fitted time-to-event survival curve. By default, it is
 #'   set to \code{TRUE}.
@@ -54,6 +57,27 @@
 #' If the spline option is chosen, the \code{knots} and \code{scale}
 #' will be included in the list of results.
 #'
+#' If the cox model option is chosen, the list of results will include
+#' \code{model}, \code{theta}, \code{vtheta}, \code{aic}, \code{bic}, and
+#' \code{piecewiseSurvivalTime}. Here
+#' \deqn{\theta = (\log(\lambda_1), \ldots, \log(\lambda_M), \beta^T)^T,}
+#' \eqn{M} denotes the number of distinct observed event times,
+#' \eqn{t_1 < \cdots < t_M},
+#' \eqn{\lambda_j} denotes the estimated hazard rate in the \eqn{j}th
+#' event time interval, \eqn{(t_{j-1}, t_j]}, and
+#' \eqn{\beta} represents the regression
+#' coefficients (log hazard ratios) from the Cox model.
+#' For a fair comparison, the estimation of baseline hazards is
+#' incorporated into the \code{aic} and \code{bic} values.
+#' In addition, \eqn{\mbox{piecewiseSurvivalTime} = (0, t_1, \ldots, t_M)}.
+#' To extend the survival curve
+#' beyond the last observed event time, a weighted average of the hazard
+#' rates from the final \code{m} event time intervals is used.
+#' The weights are proportional to the lengths of those intervals, i.e.,
+#' \deqn{\lambda_{M+1} = \sum_{j=M-m+1}^{M} w_j \lambda_j,}
+#' where \eqn{w_j = (t_j - t_{j-1})/(t_M - t_{M-m})} for
+#' \eqn{j=M-m+1,\ldots,M}.
+#'
 #' When fitting the event model by treatment, the outcome is presented
 #' as a list of lists, where each list element corresponds to a
 #' specific treatment group.
@@ -70,15 +94,16 @@
 #'
 #' @examples
 #'
-#' event_fit <- fitEvent(df = interimData2,
-#'                       event_model = "piecewise exponential",
-#'                       piecewiseSurvivalTime = c(0, 180))
+#' event_fit <- fitEvent(
+#'   df = interimData2,
+#'   event_model = "piecewise exponential",
+#'   piecewiseSurvivalTime = c(0, 180))
 #'
 #' @export
 #'
 fitEvent <- function(df, event_model = "model averaging",
                      piecewiseSurvivalTime = 0,
-                     k = 0, scale = "hazard",
+                     k = 0, scale = "hazard", m = 5,
                      showplot = TRUE, by_treatment = FALSE,
                      covariates = NULL) {
 
@@ -87,7 +112,7 @@ fitEvent <- function(df, event_model = "model averaging",
   erify::check_content(tolower(event_model),
                        c("exponential", "weibull", "log-logistic",
                          "log-normal", "piecewise exponential",
-                         "model averaging", "spline"))
+                         "model averaging", "spline", "cox model"))
 
   if (piecewiseSurvivalTime[1] != 0) {
     stop("piecewiseSurvivalTime must start with 0");
@@ -99,6 +124,7 @@ fitEvent <- function(df, event_model = "model averaging",
 
   erify::check_n(k, zero = TRUE)
   erify::check_content(tolower(scale), c("hazard", "odds", "normal"))
+  erify::check_n(m)
 
   erify::check_bool(showplot)
   erify::check_bool(by_treatment)
@@ -116,20 +142,18 @@ fitEvent <- function(df, event_model = "model averaging",
     formula = survival::Surv(time, event) ~ 1
   }
 
-
-  df <- dplyr::as_tibble(df)
-  names(df) <- tolower(names(df))
+  dt <- data.table::setDT(data.table::copy(df))
 
   if (by_treatment) {
-    ngroups = length(table(df$treatment))
+    ngroups = dt[, data.table::uniqueN(get("treatment"))]
 
-    if (!("treatment_description" %in% names(df))) {
-      df <- df %>% dplyr::mutate(
-        treatment_description = paste("Treatment", .data$treatment))
+    if (!("treatment_description" %in% names(dt))) {
+      dt[, `:=`(treatment_description =
+                  paste("Treatment", get("treatment")))]
     }
   } else {
     ngroups = 1
-    df <- df %>% dplyr::mutate(treatment = 1)
+    dt[, `:=`(treatment = 1)]
   }
 
   if (ngroups == 1) {
@@ -141,19 +165,19 @@ fitEvent <- function(df, event_model = "model averaging",
   event_fit <- list()
 
   for (i in 1:ngroups) {
-    df1 <- df %>% dplyr::filter(.data$treatment == i)
+    df1 <- dt[get("treatment") == i]
 
     n0 = nrow(df1)
-    d0 = sum(df1$event)
-    ex0 = sum(df1$time)
+    d0 = df1[, sum(get("event"))]
+    ex0 = df1[, sum(get("time"))]
 
     x = model.matrix(formula, df1)
     q = ncol(x) - 1
 
     kmfit <- survival::survfit(survival::Surv(time, event) ~ 1, data = df1)
-    kmdf <- dplyr::tibble(time = kmfit$time, surv = kmfit$surv)
-    kmdf <- dplyr::tibble(time = 0, surv = 1) %>%
-      dplyr::bind_rows(kmdf)
+    kmdf <- data.table(time = kmfit$time, surv = kmfit$surv)
+    df0 <- data.table(time = 0, surv = 1)
+    kmdf <- data.table::rbindlist(list(df0, kmdf), use.names = TRUE)
 
     if (tolower(event_model) == "exponential") {
       erify::check_positive(d0 - q, supplement = paste(
@@ -175,11 +199,9 @@ fitEvent <- function(df, event_model = "model averaging",
       # fitted survival curve
       rate = exp(as.numeric(x %*% fit2$theta))
 
-      dffit2 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(pexp(t, rate, lower.tail = FALSE))))
-
+      dffit2 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(pexp(t, rate, lower.tail = FALSE))))]
     } else if (tolower(event_model) == "weibull") {
       erify::check_positive(d0 - q - 1, supplement = paste(
         "The number of events must be >=", q+2,
@@ -191,7 +213,7 @@ fitEvent <- function(df, event_model = "model averaging",
       reg <- survival::survreg(formula, data = df1, dist = "weibull")
 
       # weibull$shape = 1/reg$scale, weibull$scale = exp(reg$coefficients)
-      # we define theta = (log(weibull$scale), -log(weibull$shape))
+      # we define theta = c(log(weibull$scale), -log(weibull$shape))
       # reg$var is for theta = c(reg$coefficients, log(reg$scale))
       fit2 <- list(model = "Weibull",
                    theta = c(as.numeric(reg$coefficients), log(reg$scale)),
@@ -203,11 +225,9 @@ fitEvent <- function(df, event_model = "model averaging",
       shape = exp(-fit2$theta[q+2])
       scale = exp(as.numeric(x %*% fit2$theta[1:(q+1)]))
 
-      dffit2 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(pweibull(t, shape, scale, lower.tail = FALSE))))
-
+      dffit2 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(pweibull(t, shape, scale, lower.tail = FALSE))))]
     } else if (tolower(event_model) == "log-logistic") {
       erify::check_positive(d0 - q - 1, supplement = paste(
         "The number of events must be >=", q+2,
@@ -230,11 +250,9 @@ fitEvent <- function(df, event_model = "model averaging",
       location = as.numeric(x %*% fit2$theta[1:(q+1)])
       scale = exp(fit2$theta[q+2])
 
-      dffit2 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(plogis(log(t), location, scale, lower.tail = FALSE))))
-
+      dffit2 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(plogis(log(t), location, scale, lower.tail = FALSE))))]
     } else if (tolower(event_model) == "log-normal") {
       erify::check_positive(d0 - q - 1, supplement = paste(
         "The number of events must be >=", q + 2,
@@ -256,14 +274,13 @@ fitEvent <- function(df, event_model = "model averaging",
       meanlog = as.numeric(x %*% fit2$theta[1:(q+1)])
       sdlog = exp(fit2$theta[q+2])
 
-      dffit2 <- dplyr::tibble(
-        time = seq(0, max(df1$time)),
-        surv = sapply(.data$time, function(t)
-          mean(plnorm(t, meanlog, sdlog, lower.tail = FALSE))))
-
+      dffit2 <- data.table(time = seq(0, max(df1$time)))[
+        , `:=`(surv = sapply(get("time"), function(t)
+          mean(plnorm(t, meanlog, sdlog, lower.tail = FALSE))))]
     } else if (tolower(event_model) == "piecewise exponential") {
-      # lambda_0(t) = lambda[j] for ucut[j] < t <= ucut[j+1], j = 1,...,J
-      # where ucut[1]=0 < ucut[2] < ... < ucut[J] < ucut[J+1]=Inf are the knots
+      # lambda_0(t) = lambda[j] for ucut[j] <= t < ucut[j+1], j = 1,...,J
+      # where ucut[1]=0 < ucut[2] < ... < ucut[J] < ucut[J+1]=Inf are
+      # the knots
       J = length(piecewiseSurvivalTime)
 
       erify::check_positive(d0 - J - q + 1, supplement = paste(
@@ -278,11 +295,10 @@ fitEvent <- function(df, event_model = "model averaging",
 
       surv = purrr::map(1:n0, function(l)
         ppwexp(time, fit2$theta, J, fit2$piecewiseSurvivalTime,
-               q, x[l,-1], lower.tail = FALSE))
+               q, x[l,], lower.tail = FALSE))
       surv = apply(matrix(purrr::list_c(surv), ncol = n0), 1, mean)
 
-      dffit2 <- dplyr::tibble(time, surv)
-
+      dffit2 <- data.table(time, surv)
     } else if (tolower(event_model) == "model averaging") {
       erify::check_positive(d0 - q - 1, supplement = paste(
         "The number of events must be >=", q + 2,
@@ -318,11 +334,10 @@ fitEvent <- function(df, event_model = "model averaging",
       time = seq(0, max(df1$time))
 
       surv = purrr::map(1:n0, function(l)
-        pmodavg(time, fit2$theta, w1, q, x[l,-1], lower.tail = FALSE))
+        pmodavg(time, fit2$theta, w1, q, x[l,], lower.tail = FALSE))
       surv = apply(matrix(purrr::list_c(surv), ncol = n0), 1, mean)
 
-      dffit2 <- dplyr::tibble(time, surv)
-
+      dffit2 <- data.table(time, surv)
     } else if (tolower(event_model) == "spline") {
       erify::check_positive(d0 - k - q - 1, supplement = paste(
         "The number of events must be >=", k + q + 2,
@@ -358,7 +373,79 @@ fitEvent <- function(df, event_model = "model averaging",
           lower.tail = FALSE)
       }
 
-      dffit2 <- dplyr::tibble(time, surv)
+      dffit2 <- data.table(time, surv)
+    } else if (tolower(event_model) == "cox model") {
+      erify::check_positive(d0 - q, supplement = paste(
+        "The number of events must be >=", q + 1,
+        "to fit a Cox model."))
+
+      erify::check_positive(d0 - m + 1, supplement = paste(
+        "m must be <= the observed number of events", d0))
+
+      reg <- phregr(df1, time = "time", event = "event",
+                    covariates = covariates)
+
+      bh <- data.table::setDT(reg$basehaz)[get("nevent") > 0]
+      d <- bh$nevent
+      llik <- reg$sumstat$loglik1 + sum(d*(log(d) - 1))
+
+      haz <- bh$haz
+      vhaz <- bh$varhaz
+
+      if (q > 0) {
+        if (q == 1) {
+          ghaz <- matrix(bh$gradhaz, ncol = 1)
+        } else {
+          ghaz <- do.call(cbind, lapply(1:q, function(i)
+            bh[[paste0("gradhaz.", i)]]))
+        }
+      }
+
+      M <- nrow(bh)
+      tcut <- c(0, bh$time)
+      lambda1 <- haz/diff(tcut)
+
+      if (q > 0) {
+        theta <- c(log(lambda1), as.numeric(reg$beta))
+        vbeta <- reg$vbeta
+        dimnames(vbeta) <- NULL
+        vtheta <- matrix(0, M+q, M+q)
+        vtheta[(M+1):(M+q),(M+1):(M+q)] <- vbeta
+        vtheta[1:M,1:M] <- diag(vhaz/(haz*haz)) + ghaz %*% vbeta %*% t(ghaz)
+        vtheta[1:M,(M+1):(M+q)] <- ghaz %*% vbeta
+        vtheta[(M+1):(M+q),1:M] <- t(vtheta[1:M,(M+1):(M+q)])
+      } else {
+        theta <- log(lambda1)
+        vtheta <- diag(vhaz/(haz*haz))
+      }
+
+      # account for the estimation of baseline hazard in AIC and BIC
+      fit2 <- list(model = "Cox model",
+                   theta = theta,
+                   vtheta = vtheta,
+                   aic = -2*llik + 2*(M+q),
+                   bic = -2*llik + (M+q)*log(n0),
+                   piecewiseSurvivalTime = tcut)
+
+      # fitted survival curve
+      time = seq(0, max(df1$time))
+
+      # extrapolate beyond the last observed event time
+      lambda2 <- sum(bh$haz[(M-m+1):M])/(bh$time[M] - bh$time[M-m])
+      lambda <- c(lambda1, lambda2)
+
+      # baseline survival
+      s1 <- sapply(time, function(t)
+        ppwexp(t, log(lambda), M+1, tcut, lower.tail = FALSE))
+
+      if (q > 0) {
+        xbeta <- as.numeric(as.matrix(x[,-1]) %*% reg$beta)
+        surv <- apply(outer(s1, exp(xbeta), `^`), 1, mean)
+      } else {
+        surv <- s1
+      }
+
+      dffit2 <- data.table(time, surv)
     }
 
 
@@ -369,6 +456,8 @@ fitEvent <- function(df, event_model = "model averaging",
     } else if (tolower(fit2$model) == "spline") {
       modeltext = paste0(fit2$model, "(k = ", k, ", ", "scale = '",
                          scale, "')")
+    } else if (tolower(fit2$model) == "cox model") {
+      modeltext = paste0(fit2$model, "(m = ", m, ")")
     } else {
       modeltext = fit2$model
     }
@@ -401,16 +490,14 @@ fitEvent <- function(df, event_model = "model averaging",
           showarrow = FALSE)) %>%
       plotly::hide_legend()
 
-    if (by_treatment && ngroups > 1) {
+    if (by_treatment) {
       fittedEvent <- fittedEvent %>%
         plotly::layout(annotations = list(
           x = 0.5, y = 1,
           text = paste0("<b>", df1$treatment_description[1], "</b>"),
           xanchor = "center", yanchor = "middle", showarrow = FALSE,
           xref = 'paper', yref = 'paper'))
-    }
 
-    if (by_treatment) {
       fit2$treatment = df1$treatment[1]
       fit2$treatment_description = df1$treatment_description[1]
     }
@@ -422,13 +509,16 @@ fitEvent <- function(df, event_model = "model averaging",
 
   # ensure that the sub plots share the same x axis range
   if (by_treatment) {
-    x_range = range(df$time)
+    x_range = range(dt$time)
     for (i in 1:ngroups) {
       event_fit[[i]]$fit_plot <- event_fit[[i]]$fit_plot %>%
         plotly::layout(xaxis = list(range = x_range))
     }
   } else {
-    event_fit = list(fit = fit2, fit_plot = fittedEvent)
+    event_fit = list(fit = fit2, fit_plot = fittedEvent,
+                     kmdf = kmdf, dffit = dffit2,
+                     text = c(modeltext, aictext, bictext))
+
   }
 
   if (showplot) {
